{
    viewer {
        savingsAccount {
            feed {
                id
                __typename
                title
                detail
                postDate
                ... on TransferInEvent {
                    amount
                }
                ... on TransferOutEvent {
                    amount
                    destinationAccount {
                        name
                    }
                }
                ... on BarcodePaymentEvent {
                    amount
                }
            }
        }
    }
}
